﻿using System;

namespace TupleStudy
{
    /// <summary>
    /// Tuple 学习
    /// 元组是一种数据结构，具有特定数量和元素序列。
    /// Tuple是C# 4.0时出的新特性，.Net Framework 4.0以上版本可用。
    /// </summary>
    public class TupleStudy
    {
        #region 创建元组
        /*
          注意：仅支持1到7个元组元素，如果有8个元素或者更多，需要使用Tuple的嵌套和Rest属性去实现。 
        */

        /// <summary>
        /// 利用构造函数创建元组
        /// 灵活，没有个数等限制
        /// </summary>
        public Tuple<int> GetTuple_Constructor_One()
        {
            var tuple = new Tuple<int>(1);

            return tuple;
        }

        public Tuple<T> GetTuple_Constructor_One<T>(T tupleType)
        {
            var tuple = new Tuple<T>(tupleType);

            return tuple;
        }

        /// <summary>
        /// 利用静态 Create方法创建元组
        /// 最多支持八个元素
        /// </summary>
        public Tuple<T1, T2> GetTuple_StaticMethod_Tow<T1, T2>(T1 tupleValue1, T2 tupleValue2)
        {
            var tuple = Tuple.Create<T1, T2>(tupleValue1,tupleValue2);

            return tuple;
        }

        /// <summary>
        /// 超过8项，使用嵌套
        /// </summary>
        public Tuple<int, int, int, int, int, int, int, Tuple<int, string>> GetTuple_Constructor_Nest()
        {
            var nestTuple = new Tuple<int, string>(8, "嵌套Tuple");
            var tuple = new Tuple<int, int, int, int, int, int, int, Tuple<int, string>>(1, 2, 3, 4, 5, 6, 7, nestTuple);


            //此处不能使用Tuple.Create()方法
            return tuple;
        }
        #endregion

        #region 表示一组数据

        /// <summary>
        /// 返回一组数据
        /// </summary>
        public Tuple<int, string, uint> GetStudentTuple(int id=1,string name="wanggaofeng", uint age=18)
        {
            var tuple = Tuple.Create<int, string, uint>(id, name, age);
            return tuple;
        }
        #endregion

        #region 从方法返回多个值

        /// <summary>
        /// 方法返回多值
        /// </summary>
        public Tuple<int, string, int, decimal> GetEgg()
        {
            var tuple = Tuple.Create<int, string, int, decimal>(1, "白色金蛋", 2018001, new decimal(25.50));

            return tuple;
        }
        #endregion

        #region 用于单参数方法的多值传递
        /// <summary>
        /// 砸金蛋
        /// 单参数方法的多值传递
        /// </summary>
        public bool SmashEgg(object egg)
        {
            var tuple = egg as Tuple<int, string, int, decimal>;
            if (tuple == null)
            {
               return false;
            }

            if (tuple.Item1 % 2 ==0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        #endregion

        #region 缺点
        // 访问元素的时候只能通过ItemX去访问，使用前需要明确元素顺序，属性名字没有实际意义，不方便记忆；

        // 最多有八个元素，要想更多只能通过最后一个元素进行嵌套扩展；

        // Tuple是一个引用类型，不像其它的简单类型一样是值类型，它在堆上分配空间，在CPU密集操作时可能有太多的创建和分配工作。
        #endregion
    }
}
